﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using MouseKeyboardLibrary;
using System.Threading;
using System.Diagnostics;

namespace ClickGen
{
    public partial class Form1 : Form
    {

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool SetCursorPos(int X, int Y);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool mouse_event(uint dwFlags, uint dx, uint dy, uint cButtons, uint dwExtraInfo);

        const uint MOUSEEVENTF_MOVE = 0x0001; /* mouse move */
        const uint MOUSEEVENTF_LEFTDOWN = 0x0002; /* left button down */
        const uint MOUSEEVENTF_LEFTUP = 0x0004; /* left button up */

        volatile bool stop = true;
        volatile int numberOfClicks = 0;
        int x, y, lastX, lastY, toX, toY;
        int sleepInterval = 1;

        char gkA = 'A';
        char gkB = 'B';
        char gkS = 'S';
        char gkR = 'R';
        char gkF = 'F';

        KeyboardHook keyboardHook = new KeyboardHook();
        MouseHook mouseHook = new MouseHook();

        Stopwatch swatch = new Stopwatch();

        public Form1()
        {
            InitializeComponent();
            clickClicked(true);

            txtA.Text = gkA.ToString();
            txtB.Text = gkB.ToString();
            txtS.Text = gkS.ToString();
            txtR.Text = gkR.ToString();
            txtF.Text = gkF.ToString();

            txtA.Visible = false;
            btnA.Visible = false;
            
            txtB.Visible = false;
            btnB.Visible = false;
            
            txtS.Visible = false;
            btnS.Visible = false;
            
            txtR.Visible = false;
            btnR.Visible = false;

            txtF.Visible = false;   
        }

        private void btnFire_Click(object sender, EventArgs e)
        {
            txtA.Visible = false;
            btnA.Visible = false;
            labelA.Visible = true;

            txtB.Visible = false;
            btnB.Visible = false;
            labelB.Visible = true;

            txtS.Visible = false;
            btnS.Visible = false;
            labelS.Visible = true;

            txtR.Visible = false;
            btnR.Visible = false;
            labelR.Visible = true;

            txtF.Visible = false;
            labelF.Visible = true;

            ResetCounter();
            BeginClick();
        }

        private void BeginClick()
        {
            try
            {
                //mouseHook.MouseDown -= new MouseEventHandler(mouseHook_MouseDown);

                x = int.Parse(txtX.Text.Trim());
                y = int.Parse(txtY.Text.Trim());
                StopAfterNumberOfClick = int.Parse(txtSopAfterClicks.Text.Trim());
                StopAfterNumberOfSec = int.Parse(txtStopAfterSec.Text.Trim());

                sleepInterval = int.Parse(txtInterval.Text);

                if (!string.IsNullOrEmpty(txtToX.Text.Trim()) && !string.IsNullOrEmpty(txtToY.Text.Trim()))
                {
                    toX = int.Parse(txtToX.Text.Trim());
                    toY = int.Parse(txtToY.Text.Trim());
                }
                swatch.Reset();
                swatch.Start();
                stop = false;
                lock (syncObj)
                    Monitor.PulseAll(syncObj);
                //mouse_event(MOUSEEVENTF_MOVE | MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);
            }
            catch
            {
                MessageBox.Show("eRRoR! Invalid Input Provided. Please verify your inputs.", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        object syncObj = new object();

        private void Clicker()
        {
            while (true)
            {
                if (stop == true)
                {
                    //System.Threading.Thread.Sleep(500);
                    lock (syncObj)
                        Monitor.Wait(syncObj, 500);

                }
                else
                {
                    try
                    {
                        if (StopAfterSeconds && (StopAfterNumberOfSec * 1000) <= swatch.ElapsedMilliseconds)
                        {
                            stop = true;
                            swatch.Reset();
                            continue;
                        }

                        if (!chkDrag.Checked)
                            GenerateClick();
                        else
                            GenerateDrag();

                        numberOfClicks++;
                        UpdateLable(lblClickCount, numberOfClicks.ToString());

                        if (StopAfterClicks && StopAfterNumberOfClick <= numberOfClicks)
                        {
                            stop = true;
                            continue;
                        }



                        lock (syncObj)
                            Monitor.Wait(syncObj, sleepInterval);
                        //System.Threading.Thread.Sleep(sleepInterval);                      
                    }
                    catch
                    {
                    }
                }
            }
        }

        private void GenerateClick()
        {
            SetCursorPos(x, y);
            mouse_event(MOUSEEVENTF_MOVE | MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);

            lock (syncObj)
                Monitor.Wait(syncObj, int.Parse(txtClickTime.Text));

            //System.Threading.Thread.Sleep(int.Parse(txtClickTime.Text));
            //SetCursorPos(x+100, y+100);
            mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
        }

        private void GenerateDrag()
        {
            int xDiff = -x + toX;
            int yDiff = -y + toY;

            int dist;

            if (Math.Abs(xDiff) > Math.Abs(yDiff))
                dist = Math.Abs(xDiff);
            else
                dist = Math.Abs(yDiff);

            int time = int.Parse(txtClickTime.Text);

            int steps = dist;

            decimal sleepByDist = 0;

            int jumpCounter = 0;

            do
            {
                jumpCounter++;
                steps = dist / jumpCounter;
                sleepByDist = (decimal)time / (decimal)steps;
            } while (sleepByDist < 15);

            //if(sleepByDist > 15) //must sleep at least for 15 ms. Smallest enough for smooth move. 

            //TODO: count steps by distance and time. Execute minimum
            int newX = x;
            int newY = y;
            SetCursorPos(x, y);

            mouse_event(MOUSEEVENTF_MOVE | MOUSEEVENTF_LEFTDOWN, 0, 0, 0, 0);

            decimal xDeltaBalance = 0;
            decimal yDeltaBalance = 0;

            for (int i = 1; i <= steps; i++)
            {
                int xDelta, yDelta;

                if (xDiff >= 0)
                    xDelta = (int)Math.Ceiling(xDiff / (decimal)steps);
                else
                    xDelta = (int)Math.Floor(xDiff / (decimal)steps);

                if (yDiff >= 0)
                    yDelta = (int)Math.Ceiling(yDiff / (decimal)steps);
                else
                    yDelta = (int)Math.Floor(yDiff / (decimal)steps);

                xDeltaBalance += (xDiff / (decimal)steps) - xDelta;
                yDeltaBalance += (yDiff / (decimal)steps) - yDelta;

                if (Math.Abs(xDeltaBalance) >= 1)
                {
                    if (xDelta > 0)
                    {
                        xDelta -= 1;
                        xDeltaBalance += 1;
                    }
                    else
                    {
                        xDelta += 1;
                        xDeltaBalance -= 1;
                    }

                }
                if (Math.Abs(yDeltaBalance) >= 1)
                {
                    if (yDelta > 0)
                    {
                        yDelta -= 1;
                        yDeltaBalance += 1;
                    }
                    else
                    {
                        yDelta += 1;
                        yDeltaBalance -= 1;
                    }

                }

                if ((xDiff > 0 && (newX + xDelta) < toX)
                    || (xDiff < 0 && (newX + xDelta) > toX))
                    newX += xDelta;
                else
                    newX = toX;

                if ((yDiff > 0 && (newY + yDelta) < toY)
                    || (yDiff < 0 && (newY + yDelta) > toY))
                    newY += yDelta;
                else
                    newY = toY;

                SetCursorPos(newX, newY);

                lock (syncObj)
                    Monitor.Wait(syncObj, (int)sleepByDist);

                //Thread.Sleep((int)sleepByDist);
            }
            SetCursorPos(toX, toY);
            mouse_event(MOUSEEVENTF_LEFTUP, 0, 0, 0, 0);
        }


        private void UpdateLable(Label l, string text)
        {
            if (l.InvokeRequired)
                this.Invoke(new MethodInvoker(() => l.Text = text));
            else
                l.Text = text;
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            keyboardHook.KeyDown += new KeyEventHandler(keyboardHook_KeyDown);
            keyboardHook.KeyUp += new KeyEventHandler(keyboardHook_KeyUp);

            mouseHook.MouseMove += new MouseEventHandler(mouseHook_MouseMove);
            mouseHook.MouseDown += new MouseEventHandler(mouseHook_MouseDown);
            mouseHook.MouseUp += new MouseEventHandler(mouseHook_MouseUp);

            mouseHook.Start();
            keyboardHook.Start();

            System.Threading.Thread t = new System.Threading.Thread(Clicker);
            t.IsBackground = true;
            t.Start();

            foreach (Control c in groupBox1.Controls)
            {
                if (c is TextBox)
                    controlOldValidVal.Add(c.Name, ((TextBox)c).Text);
            }
        }

        void keyboardHook_KeyUp(object sender, KeyEventArgs e)
        {
            if (F_KeyFeature == true)
                stop = true;
            flock = false;
        }

        bool flock = false;

        void keyboardHook_KeyDown(object sender, KeyEventArgs e)
        {
            if (txtA.Focused || txtB.Focused || txtF.Focused || txtR.Focused || txtS.Focused)
                return;

            if (e.KeyCode.ToString().Equals(gkS.ToString(), StringComparison.InvariantCultureIgnoreCase))
            {
                stop = true;
                lock (syncObj)
                    Monitor.PulseAll(syncObj);
                //mouseHook.MouseDown += new MouseEventHandler(mouseHook_MouseDown);
                //lblClickCount.Text = numberOfClicks.ToString();

            }
            /*AddKeyboardEvent(
                "KeyDown",
                e.KeyCode.ToString(),
                "",
                e.Shift.ToString(),
                e.Alt.ToString(),
                e.Control.ToString()
                );*/
            if (e.KeyCode.ToString().Equals(gkR.ToString(), StringComparison.InvariantCultureIgnoreCase))
                ResetCounter();

            if (F_KeyFeature && e.KeyCode.ToString().Equals(gkF.ToString(), StringComparison.InvariantCultureIgnoreCase))
            {
                if (!flock)
                {
                    flock = true;
                    BeginClick();
                }
            }
            else
                flock = false;

            if (e.KeyCode.ToString().Equals(gkA.ToString(), StringComparison.InvariantCultureIgnoreCase))
            {
                txtX.Text = lastX.ToString();
                txtY.Text = lastY.ToString();
            }
            if (e.KeyCode.ToString().Equals(gkB.ToString(), StringComparison.InvariantCultureIgnoreCase))
            {
                txtToX.Text = lastX.ToString();
                txtToY.Text = lastY.ToString();
                chkDrag.Checked = true;
            }

        }

        void ResetCounter()
        {
            numberOfClicks = 0;
            lblClickCount.Text = "0";
        }

        void mouseHook_MouseMove(object sender, MouseEventArgs e)
        {
            //lblClickCount.Text = numberOfClicks.ToString();

            AddMouseEvent(
                "MouseMove",
                e.Button.ToString(),
                e.X.ToString(),
                e.Y.ToString(),
                ""
                );
        }

        void AddMouseEvent(string eventType, string button, string x, string y, string delta)
        {
            int xInt = int.Parse(x);
            int yInt = int.Parse(y);
            if (lastX == xInt && lastY == yInt)
                return; //to ignore during fast click generation.

            lastX = int.Parse(x);
            lastY = int.Parse(y);

            if (chkPointToShoot.Checked)
            {
                txtX.Text = x;
                txtY.Text = y;
            }
            lblCurrent.Text = ("Your current mouse co-ordinates are:\r\n X: " + x + " and Y: " + y + ". See right side panel for help ->");
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            keyboardHook.KeyDown -= new KeyEventHandler(keyboardHook_KeyDown);
            mouseHook.MouseMove -= new MouseEventHandler(mouseHook_MouseMove);
            keyboardHook.Stop();
            mouseHook.Stop();
        }


        private void clickClicked(bool generateClick)
        {
            txtToX.Enabled = !generateClick;
            txtToY.Enabled = !generateClick;

            chkPointToShoot.Checked = false;
            chkPointToShoot.Enabled = generateClick;
            chkClicksOnF.Checked = false;
            chkClicksOnF.Enabled = generateClick;

            if (!generateClick)
            {
                lblDragFrom.Text = "Drag from: ";
                lblWaitBtn.Text = "Wait between two drags:";
                chkClicksOnF.Text = "Generate drags when ";
                lblStopAfterClicks.Text = "drags";
                lblTotalClicks.Text = "Total Drags Generated:";
            }
            else
            {
                lblDragFrom.Text = "Click at:";
                lblWaitBtn.Text = "Wait between two clicks:";
                chkClicksOnF.Text = "Generate clicks when ";
                lblStopAfterClicks.Text = "clicks";
                lblTotalClicks.Text = "Total Clicks Generated:";
            }

        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            clickClicked(!chkDrag.Checked);
            ResetCounter();
        }

        private void lnkToBlog_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://automouseclicker.blogspot.in/2013/01/auto-mouse-clicker-introduction.html");
        }



        int dragFromX, dragFromY, dragToX, dragToY;

        public void mouseHook_MouseDown(object sender, MouseEventArgs e)
        {
            return;
            if (stop)
            {
                dragFromX = e.X;
                dragFromY = e.Y;
            }
            else
            {
                dragFromX = int.MinValue;
                dragFromY = int.MinValue;
            }
        }

        public void mouseHook_MouseUp(object sender, MouseEventArgs e)
        {
            //lblClickCount.Text = numberOfClicks.ToString();

            return;

            if (stop)
            {
                dragToX = e.X;
                dragToY = e.Y;
            }
            else
            {
                dragToX = int.MinValue;
                dragToY = int.MinValue;
            }

            if (dragFromX != int.MinValue && dragFromY != int.MinValue && dragToX != int.MinValue && dragToY != int.MinValue
                && (dragFromX != dragToX || dragFromY != dragToY))
            {
                txtX.Text = dragFromX.ToString();
                txtY.Text = dragFromY.ToString();

                txtToX.Text = dragToX.ToString();
                txtToY.Text = dragToY.ToString();

                chkDrag.Checked = true;
            }
        }

        bool F_KeyFeature = false;
        bool StopAfterClicks = false;
        bool StopAfterSeconds = false;
        int StopAfterNumberOfClick = 0;
        int StopAfterNumberOfSec = 0;

        private void chkClicksOnF_CheckedChanged(object sender, EventArgs e)
        {
            chkPointToShoot.Checked = false;
            chkPointToShoot.Visible = chkClicksOnF.Checked;
            F_KeyFeature = chkClicksOnF.Checked;
            btnFire.Enabled = !chkClicksOnF.Checked;

            chkStopAfterClick.Checked = !chkClicksOnF.Checked;
            chkStopAfterSec.Checked = !chkClicksOnF.Checked;


            chkStopAfterClick.Visible = !chkClicksOnF.Checked;
            chkStopAfterSec.Visible = !chkClicksOnF.Checked;
            txtSopAfterClicks.Visible = !chkClicksOnF.Checked;
            txtStopAfterSec.Visible = !chkClicksOnF.Checked;
            lblStopAfterClicks.Visible = !chkClicksOnF.Checked;
            lblStopAfterSeconds.Visible = !chkClicksOnF.Checked;
        }


        bool chkBoxSyncCall = false;
        private void chkStopAfterClick_CheckedChanged(object sender, EventArgs e)
        {
            if (chkBoxSyncCall == false)
            {
                chkBoxSyncCall = true;
                chkStopAfterSec.Checked = false;
            }
            StopAfterClicks = chkStopAfterClick.Checked;
            txtSopAfterClicks.Enabled = StopAfterClicks;
            chkBoxSyncCall = false;
            //UpdateStopAfterCheckBox(chkStopAfterClick.Checked, false);
        }

        private void UpdateStopAfterCheckBox(bool isStopAfterClicks, bool isStopAfterSecs)
        {
            StopAfterClicks = isStopAfterClicks;


        }

        private void chkStopAfterSec_CheckedChanged(object sender, EventArgs e)
        {
            if (chkBoxSyncCall == false)
            {
                chkBoxSyncCall = true;
                chkStopAfterClick.Checked = false;
            }
            StopAfterSeconds = chkStopAfterSec.Checked;
            txtStopAfterSec.Enabled = StopAfterSeconds;
            chkBoxSyncCall = false;
            //UpdateStopAfterCheckBox(false, chkStopAfterClick.Checked);

        }


        Dictionary<string, string> controlOldValidVal = new Dictionary<string, string>();
        private void txt_TextChanged_ForceInt(object sender, EventArgs e)
        {
            TextBox tb = (TextBox)sender;

            int x;
            try
            {
                if (!int.TryParse(tb.Text.Trim(), out x) && tb.Text.Trim() != "")
                    tb.Text = controlOldValidVal[tb.Name];
                else
                {
                    controlOldValidVal[tb.Name] = tb.Text;
                }
            }
            catch
            {
                // throw;
            }

        }

        private void chkPointToShoot_CheckedChanged(object sender, EventArgs e)
        {
            txtClickTime.Enabled = !chkPointToShoot.Checked;
            if (chkPointToShoot.Checked)
                txtClickTime.Text = "0";
        }

        private void txtF_Leave(object sender, EventArgs e)
        {
           
        }

        private void labelA_Click(object sender, EventArgs e)
        {
            txtA.Visible = true;
            btnA.Visible = true;
            labelA.Visible = false;
        }

        private void labelB_Click(object sender, EventArgs e)
        {
            txtB.Visible = true;
            btnB.Visible = true;
            labelB.Visible = false;
        }

        private void labelS_Click(object sender, EventArgs e)
        {
            txtS.Visible = true;
            btnS.Visible = true;
            labelS.Visible = false;
        }

        private void labelR_Click(object sender, EventArgs e)
        {
            txtR.Visible = true;
            btnR.Visible = true;
            labelR.Visible = false;
        }

        private void btnA_Click(object sender, EventArgs e)
        {
            TextBox tx = txtA;
            Label lb1 = labelA;
            Label lb2 = labelAA;

            if (string.IsNullOrEmpty(tx.Text))
            {
                tx.Text = gkA.ToString();
                return;
            }

            gkA = tx.Text[0];
            lb1.Text = gkA.ToString();
            lb2.Text = gkA.ToString();
            tx.Visible = false; 
            lb1.Visible=true;
            btnA.Visible = false;

        }

        private void btnB_Click(object sender, EventArgs e)
        {
            TextBox tx = txtB;
            Label lb1 = labelB;
            Label lb2 = labelBB;

            if (string.IsNullOrEmpty(tx.Text))
            {
                tx.Text = gkB.ToString();
                return;
            }

            gkB = tx.Text[0];
            lb1.Text = gkB.ToString();
            lb2.Text = gkB.ToString();
            tx.Visible = false; 
            lb1.Visible=true;
            btnB.Visible = false;

        }

        private void btnS_Click(object sender, EventArgs e)
        {
            TextBox tx = txtS;
            Label lb1 = labelS;
            Label lb2 = labelSS;

            if (string.IsNullOrEmpty(tx.Text))
            {
                tx.Text = gkS.ToString();
                return;
            }

            gkS = tx.Text[0];
            lb1.Text = gkS.ToString();
            lb2.Text = gkS.ToString();
            tx.Visible = false; 
            lb1.Visible=true;
            btnS.Visible = false;
        }

        private void btnR_Click(object sender, EventArgs e)
        {
            TextBox tx = txtR;
            Label lb1 = labelR;
            Label lb2 = labelRR;

            if (string.IsNullOrEmpty(tx.Text))
            {
                tx.Text = gkR.ToString();
                return;
            }

            gkR = tx.Text[0];
            lb1.Text = gkR.ToString();
            lb2.Text = gkR.ToString();
            tx.Visible = false; 
            lb1.Visible=true;
            btnR.Visible = false;
        }

        private void txtF_TextChanged(object sender, EventArgs e)
        {
            txtF.SelectionStart = 0;
            txtF.SelectionLength = txtF.Text.Length;
            TextBox tx = txtF;
            Label lb1 = labelF;
            Label lb2 = labelFF;

            if (string.IsNullOrEmpty(tx.Text))
            {
                tx.Text = gkF.ToString();
                return;
            }

            gkF = tx.Text[0];
            lb1.Text = gkF.ToString();
            lb2.Text = gkF.ToString();
            txtF.Visible = false;
            labelF.Visible = true;
        }

        private void lblF_Click(object sender, EventArgs e)
        {
            txtF.SelectionStart = 0;
            txtF.SelectionLength = txtF.Text.Length;
            txtF.Visible = true;
            labelF.Visible = false;
        }
    }
}
